@@ -1,5 +1,8 @@
 package ca.ubc.cs.beta.aclib.algorithmrun;
 
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
 import ca.ubc.cs.beta.aclib.algorithmrun.kill.KillableAlgorithmRun;
 import ca.ubc.cs.beta.aclib.exceptions.IllegalWrapperOutputException;
 import ca.ubc.cs.beta.aclib.execconfig.AlgorithmExecutionConfig;
@@ -12,8 +15,9 @@ import ca.ubc.cs.beta.aclib.runconfig.RunConfig;
  * @author seramage
  *
  */
-public abstract class AbstractAlgorithmRun implements Runnable, AlgorithmRun{
-
+public abstract class AbstractAlgorithmRun implements Runnable, AlgorithmRun
+{
+	
 	/**
 	 * 
 	 */
@@ -30,7 +34,7 @@ public abstract class AbstractAlgorithmRun implements Runnable, AlgorithmRun{
 	private double runtime;
 	private double runLength;
 	private double quality;
-	private long resultSeed; 
+	 
 	
 	/**
 	 * Raw result line reported by the target algorithm (potentially useful if the result line is corrupt)
@@ -64,6 +68,9 @@ public abstract class AbstractAlgorithmRun implements Runnable, AlgorithmRun{
 	 */
 	private String additionalRunData = "";
 	
+	
+	private static final Logger log = LoggerFactory.getLogger(AbstractAlgorithmRun.class);
+	
 	/**
 	 * Sets the values for this Algorithm Run
 	 * @param acResult					The result of the Run
@@ -148,7 +155,6 @@ public abstract class AbstractAlgorithmRun implements Runnable, AlgorithmRun{
 		this.runtime = Math.min(runtime, Double.MAX_VALUE);
 		this.runLength = Math.min(runLength, Double.MAX_VALUE);
 		this.quality = quality;
-		this.resultSeed = resultSeed;
 		
 		if(this.saveRawResultLine())
 		{
@@ -157,11 +163,16 @@ public abstract class AbstractAlgorithmRun implements Runnable, AlgorithmRun{
 		
 		this.runResultWellFormed = runResultWellFormed;
 
-		this.additionalRunData = additionalRunData;
+	
+		
 		if(this.additionalRunData == null)
 		{
 			throw new IllegalArgumentException("Additional Run Data cannot be NULL");
+		} else
+		{
+			this.additionalRunData = additionalRunData.replace("\n","\\n").replace(',', ';');
 		}
+		
 		this.resultSet = true;
 		if(!(this instanceof KillableAlgorithmRun))
 		{
@@ -250,7 +261,7 @@ public abstract class AbstractAlgorithmRun implements Runnable, AlgorithmRun{
 	@Override
 	public final long getResultSeed() {
 		if(!isRunResultWellFormed()) throw new IllegalStateException("Execution Result was not well formed");
-		return resultSeed;
+		return this.getRunConfig().getProblemInstanceSeedPair().getSeed();
 	}
 	
 	private final String _getResultLine()
@@ -364,6 +375,11 @@ public abstract class AbstractAlgorithmRun implements Runnable, AlgorithmRun{
 		return additionalRunData;
 	}
 	
+	@Override
+	public boolean isCensoredEarly()
+	{
+		return ((getRunResult().equals(RunResult.TIMEOUT) && getRunConfig().hasCutoffLessThanMax()) ||  (getRunResult().equals(RunResult.KILLED) && getRuntime() < getRunConfig().getCutoffTime()));
+	}
 	protected boolean isResultSet()
 	{
 		return resultSet;
@@ -373,6 +389,6 @@ public abstract class AbstractAlgorithmRun implements Runnable, AlgorithmRun{
 	{
 		return false;
 	}
-	
+
 	
 }
