@@ -11,16 +11,16 @@ public class SMACNoArgHandler implements NoArgumentHandler{
 		sb.append("SMAC (http://www.cs.ubc.ca/labs/beta/Projects/SMAC/) is an automatic configurator that allows users to automatically tune algorithm configuration spaces.").append("\n\n");
 
 		sb.append("  Basic Usage:\n");
-		sb.append("  smac --scenarioFile <file> --seed 1\n\n");
+		sb.append("  smac --scenario-file <file> \n\n");
 	
 		sb.append("  Skipping Validation:\n");
-		sb.append("  smac --scenarioFile <file> --seed 1  --doValidation false\n\n");
+		sb.append("  smac --scenario-file <file> --validation false\n\n");
 
 		sb.append("  Linux Example:\n");
-		sb.append("  ./smac --scenario-file ./example_scenarios/saps/saps-scenario.txt  --num-run 1\n\n");
+		sb.append("  ./smac --scenario-file ./example_scenarios/saps/saps-scenario.txt\n\n");
 		sb.append("  Windows Example:\n");
 		
-		sb.append("  smac.bat --scenario-file example_scenarios\\saps\\saps-scenario-windows.txt --num-run 1\n\n");
+		sb.append("  smac.bat --scenario-file example_scenarios\\saps\\saps-scenario-windows.txt \n\n");
 		
 		sb.append("  Full version information is available with :\n");
 		sb.append("  smac -v\n\n");
