@@ -15,7 +15,11 @@ import java.util.Arrays;
 import java.util.Collections;
 import java.util.Map.Entry;
 import java.util.Queue;
+import java.util.Set;
+import java.util.TreeSet;
+import java.util.UUID;
 import java.util.concurrent.BlockingQueue;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Semaphore;
@@ -41,6 +45,7 @@ import ca.ubc.cs.beta.aclib.algorithmrun.kill.KillableWrappedAlgorithmRun;
 import ca.ubc.cs.beta.aclib.concurrent.threadfactory.SequentiallyNamedThreadFactory;
 import ca.ubc.cs.beta.aclib.configspace.ParamConfiguration.StringFormat;
 import ca.ubc.cs.beta.aclib.execconfig.AlgorithmExecutionConfig;
+import ca.ubc.cs.beta.aclib.misc.associatedvalue.Pair;
 import ca.ubc.cs.beta.aclib.misc.logback.MarkerFilter;
 import ca.ubc.cs.beta.aclib.misc.logging.LoggingMarker;
 import ca.ubc.cs.beta.aclib.misc.string.SplitQuotedString;
@@ -61,13 +66,19 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 	/**
 	 * Regex that we hope to match
 	 */
-	public static final String AUTOMATIC_CONFIGURATOR_RESULT_REGEX = "^\\s*(Final)?\\s*[Rr]esult\\s+(?:([Ff]or)|([oO]f))\\s+(?:(HAL)|(ParamILS)|(SMAC)|([tT]his [wW]rapper)):";
 	
+	
+	//maybe merge these one day
+	public static final String AUTOMATIC_CONFIGURATOR_RESULT_REGEX = "^\\s*Result\\s*of\\s*(this)?\\s*[Aa]lgorithm\\s*[rR]un\\s*:";
+	
+	public static final String OLD_AUTOMATIC_CONFIGURATOR_RESULT_REGEX = "^\\s*(Final)?\\s*[Rr]esult\\s+(?:([Ff]or)|([oO]f))\\s+(?:(HAL)|(ParamILS)|(SMAC)|([tT]his [wW]rapper)):";
 	/**
 	 * Compiled REGEX
 	 */
 	private static final Pattern pattern = Pattern.compile(AUTOMATIC_CONFIGURATOR_RESULT_REGEX);
 	
+	private static final Pattern oldPattern = Pattern.compile(OLD_AUTOMATIC_CONFIGURATOR_RESULT_REGEX);
+	
 	private static transient Logger log = LoggerFactory.getLogger(CommandLineAlgorithmRun.class);
 	
 	
@@ -83,32 +94,61 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 	 */
 	private transient KillHandler killHandler;
 	
-	public static final String PORT_ENVIRONMENT_VARIABLE = "ACLIB_PORT";
-	public static final String FREQUENCY_ENVIRONMENT_VARIABLE = "ACLIB_CPU_TIME_FREQUENCY";
-	public static final String CONCURRENT_TASK_ID = "ACLIB_CONCURRENT_TASK_ID";
+	public static final String PORT_ENVIRONMENT_VARIABLE = "AEATK_PORT";
+	public static final String FREQUENCY_ENVIRONMENT_VARIABLE = "AEATK_CPU_TIME_FREQUENCY";
+	public static final String CONCURRENT_TASK_ID = "AEATK_CONCURRENT_TASK_ID";
+	
+	
+	
+	/**
+	 * This variable is public only for unit test purposes,
+	 * this is not guaranteed to be the actual environment variable of child processes
+	 */
+	public static final String EXECUTION_UUID_ENVIRONMENT_VARIABLE_DEFAULT = "AEATK_EXECUTION_UUID"; 
+	
+	
+	/**
+	 * Stores a unique UUID for the run, used in environment variables.
+	 */
+	private final UUID uuid = UUID.randomUUID();
+	
+	static 
+	{
+		if(!System.getenv().containsKey(EXECUTION_UUID_ENVIRONMENT_VARIABLE_DEFAULT))
+		{
+			envVariableForChildren = EXECUTION_UUID_ENVIRONMENT_VARIABLE_DEFAULT;
+		} else
+		{
+			int i=0;
+			while( System.getenv().containsKey(EXECUTION_UUID_ENVIRONMENT_VARIABLE_DEFAULT + "_SUB_" + (i++)));
+			envVariableForChildren = EXECUTION_UUID_ENVIRONMENT_VARIABLE_DEFAULT + "_SUB_" + (i++);
+		}
+	}
+	
+	private static final String envVariableForChildren;
+	
 	
 	/**
 	 * Marker for logging
 	 */
 	private static transient Marker fullProcessOutputMarker = MarkerFactory.getMarker(LoggingMarker.FULL_PROCESS_OUTPUT.name());
 	
-	private static String commandSeparator = ";";
-	
-	
-	
+	public final static String COMMAND_SEPERATOR;
+
 	static {
-		log.warn("This version of SMAC hardcodes run length for calls to the target algorithm to {}.", Integer.MAX_VALUE);
+		log.trace("This version of SMAC hardcodes run length for calls to the target algorithm to {}.", Integer.MAX_VALUE);
 		
 		if(System.getProperty("os.name").toLowerCase().contains("win"))
 		{
-			commandSeparator = "&";
+			COMMAND_SEPERATOR = "&";
+		} else
+		{
+			COMMAND_SEPERATOR = ";";
 		}
 		
 	}
 	
-	private static final double WALLCLOCK_TIMING_SLACK = 0.001;
-	
-	private transient ExecutorService threadPoolExecutor = Executors.newCachedThreadPool(new SequentiallyNamedThreadFactory("Command Line Target Algorithm Evaluator Thread ")); 
+	//private transient
 	
 	private final int observerFrequency;
 	
@@ -131,14 +171,47 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 	 * @param runConfig			run configuration we are executing
 	 * @param executionIDs 
 	 */
+	
+	
+	private static transient final AtomicBoolean jvmShutdownDetected = new AtomicBoolean(false);
+	
+	private static final Set<Pair<CommandLineAlgorithmRun, Process>> outstandingRuns  = Collections.newSetFromMap(new ConcurrentHashMap<Pair<CommandLineAlgorithmRun, Process>,Boolean>());
+	static
+	{
+		Thread shutdownThread = new Thread(new Runnable()
+		{
+
+			@Override
+			public void run() {
+				Thread.currentThread().setName("CLI Shutdown Thread");
+				jvmShutdownDetected.set(true);
+				if(outstandingRuns.size() > 0)
+				{
+					log.debug("Terminating approximately {} outstanding algorithm runs", outstandingRuns.size());
+				}
+				log.trace("Further runs will be instantly terminated");
+				for(Pair<CommandLineAlgorithmRun, Process> p : outstandingRuns)
+				{
+					p.getFirst().killProcess(p.getSecond());
+				}
+			}
+			
+		});
+		
+		log.trace("Shutdown hook to terminate all outstanding runs enabled");
+		Runtime.getRuntime().addShutdownHook(shutdownThread);
+	}
+	
+	
+	
+	
 	public CommandLineAlgorithmRun(AlgorithmExecutionConfig execConfig, RunConfig runConfig, TargetAlgorithmEvaluatorRunObserver obs, KillHandler handler, CommandLineTargetAlgorithmEvaluatorOptions options, BlockingQueue<Integer> executionIDs) 
 	{
 		super(execConfig, runConfig);
-		//TODO Test
 		if(runConfig.getCutoffTime() <= 0 || handler.isKilled())
 		{
 			
-			log.info("Cap time is less than or equal to zero for {} setting run as timeout", runConfig);
+			log.trace("Cap time is less than or equal to zero for {} setting run as timeout", runConfig);
 			String rawResultLine = "[DIDN'T BOTHER TO RUN ALGORITHM AS THE CAPTIME IS NOT POSITIVE]";
 			
 			this.setResult(RunResult.TIMEOUT, 0, 0, 0, runConfig.getProblemInstanceSeedPair().getSeed(), rawResultLine,"");
@@ -155,6 +228,9 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 		
 		this.options = options;
 		this.executionIDs = executionIDs;
+		
+	
+		
 	}
 	
 	private static final int MAX_LINES_TO_SAVE = 1000;
@@ -172,13 +248,25 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 		//Notify observer first to trigger kill handler
 		runObserver.currentStatus(Collections.singletonList((KillableAlgorithmRun) new RunningAlgorithmRun(execConfig, getRunConfig(),  0,  0,0, getRunConfig().getProblemInstanceSeedPair().getSeed(), 0, killHandler)));
 		
+		
+		if(jvmShutdownDetected.get())
+		{
+			
+			String rawResultLine = "JVM Shutdown Detected";
+			
+			this.setResult(RunResult.KILLED, 0, 0, 0, runConfig.getProblemInstanceSeedPair().getSeed(), rawResultLine,"JVM Shutdown Detected, algorithm not executed");
+			runObserver.currentStatus(Collections.singletonList(new KillableWrappedAlgorithmRun(this)));
+			return;
+		}
+		
 		if(killHandler.isKilled())
 		{
 			
-			log.debug("Run was killed", runConfig);
-			String rawResultLine = "Killed Manually";
+			log.trace("Run was killed", runConfig);
+			String rawResultLine = "Kill detected before target algorithm invoked";
 			
-			this.setResult(RunResult.KILLED, 0, 0, 0, runConfig.getProblemInstanceSeedPair().getSeed(), rawResultLine,"");
+			this.setResult(RunResult.KILLED, 0, 0, 0, runConfig.getProblemInstanceSeedPair().getSeed(), rawResultLine,"Kill detected before target algorithm invoked");
+			runObserver.currentStatus(Collections.singletonList(new KillableWrappedAlgorithmRun(this)));
 			return;
 		}
 		
@@ -204,16 +292,16 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 				//Check kill handler again
 				if(killHandler.isKilled())
 				{
+					log.trace("Run was killed", runConfig);
+					String rawResultLine = "Kill detected before target algorithm invoked";
 					
-					log.debug("Run was killed", runConfig);
-					String rawResultLine = "Killed Manually";
-					
-					this.setResult(RunResult.KILLED, 0, 0, 0, runConfig.getProblemInstanceSeedPair().getSeed(), rawResultLine,"");
+					this.setResult(RunResult.KILLED, 0, 0, 0, runConfig.getProblemInstanceSeedPair().getSeed(), rawResultLine,"Kill detected before target algorithm invoked");
+					runObserver.currentStatus(Collections.singletonList(new KillableWrappedAlgorithmRun(this)));;
 					return;
 				}
 				
 				
-				final AtomicInteger pid = new AtomicInteger(-1);
+				
 				int port = 0;
 				final DatagramSocket serverSocket;
 				if(options.listenForUpdates)
@@ -258,10 +346,12 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 				               
 							} catch(RuntimeException e)
 							{
-								log.debug("Got some runtime exception while processing data packet", e);
+								log.trace("Got some runtime exception while processing data packet", e);
 							} catch(SocketException e)
 							{
-								log.trace("SocketException occurred which we expected when we shutdown probably", e);
+								//Don't log this since this socket exception is what we
+								//we expect since most of the time we will be blocked on the socket
+								//when we are shutdown and interrupted.
 								return;
 							} catch (IOException e) {
 								log.warn("Unknown IOException occurred ", e);
@@ -275,14 +365,18 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 				
 				this.startWallclockTimer();
 				proc = runProcess(port, token);
-			
+				outstandingRuns.add(new Pair<CommandLineAlgorithmRun, Process>(this, proc));
+				
+				
 				
 				
 				
 				final Process innerProcess = proc; 
 				
+				
 				final Semaphore stdErrorDone = new Semaphore(0);
-			
+	
+				
 				Runnable standardErrorReader = new Runnable()
 				{
 	
@@ -344,7 +438,7 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 							{
 								
 								stdErrorDone.release();
-								log.debug("Standard Error Done");
+								log.trace("Standard Error Done");
 							}
 						} catch(InterruptedException e)
 						{
@@ -387,11 +481,8 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 								if(killHandler.isKilled())
 								{
 									wasKilled = true;
-									log.debug("Trying to kill");
+									log.trace("Trying to kill");
 									killProcess(proc);
-									//log.debug("Process destroy() called now waiting for completion");
-									
-									//log.debug("Process has exited with code {}", proc.waitFor());
 									return;
 								}
 								Thread.sleep(observerFrequency - 25);
@@ -407,75 +498,103 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 					
 				};
 				
-				
-				if(options.listenForUpdates)
+				ExecutorService threadPoolExecutor = Executors.newCachedThreadPool(new SequentiallyNamedThreadFactory("Command Line Target Algorithm Evaluator Thread "));
+				try 
 				{
-					threadPoolExecutor.execute(socketThread);
-				}
-				threadPoolExecutor.execute(observerThread);
-				threadPoolExecutor.execute(standardErrorReader);
-				BufferedReader read = new BufferedReader(new InputStreamReader(proc.getInputStream()));
-				
-				//Scanner procIn = new Scanner(proc.getInputStream());
-			
-				processRunLoop(read,proc);
-				
-				killProcess(proc);
-				
-				if(!this.isRunCompleted())
-				{
-					if(wasKilled)
+					if(options.listenForUpdates)
 					{
-						double currentTime = Math.max(0,currentRuntime.get());
-						this.setResult(RunResult.KILLED, currentTime, 0,0, getRunConfig().getProblemInstanceSeedPair().getSeed(), "Killed Manually", "" );
+						threadPoolExecutor.execute(socketThread);
+					}
+					threadPoolExecutor.execute(observerThread);
+					threadPoolExecutor.execute(standardErrorReader);
+					BufferedReader read = new BufferedReader(new InputStreamReader(proc.getInputStream()));
+					
+					//Scanner procIn = new Scanner(proc.getInputStream());
+					
+					
+					 
+					if(jvmShutdownDetected.get())
+					{ //Possible that this run started after the shutdown call was flagged, but before we put it in the map.
+					
+						killProcess(proc);
+					}
+					try
+					{
+						processRunLoop(read,proc);
+					} finally
+					{
+						killProcess(proc);
+					}
+					
+					if(!this.isRunCompleted())
+					{
+						if(wasKilled)
+						{
+							double currentTime = Math.max(0,currentRuntime.get());
+							this.setResult(RunResult.KILLED, currentTime, 0,0, getRunConfig().getProblemInstanceSeedPair().getSeed(), "Killed Manually", "" );
+							
+						} else if(jvmShutdownDetected.get())
+						{
+							double currentTime = Math.max(0,currentRuntime.get());
+							this.setResult(RunResult.KILLED, currentTime, 0,0, getRunConfig().getProblemInstanceSeedPair().getSeed(), "JVM Shutdown Detected", "" );							
+						} else
+						{
+							this.setCrashResult("ERROR: Wrapper did not output anything that matched the expected output (\"Result of algorithm run:...\"). Please try executing the wrapper directly" );
+						}
+				}
+					
+					
+					switch(this.getRunResult())
+					{
+						case ABORT:
+						case CRASHED:
+							
+								
 						
-					} else {
-						this.setCrashResult("Wrapper did not output anything that matched our regex please see the manual for more information. Please try executing the wrapper directly and ensuring that some line starts with: \"Results for ParamILS:\" (case sensitive). In more gorey detail it needs to match the following Regular Expression: " + AUTOMATIC_CONFIGURATOR_RESULT_REGEX );
+								log.error("The following algorithm call failed: cd \"{}\" " + COMMAND_SEPERATOR + "  {} ",new File(execConfig.getAlgorithmExecutionDirectory()).getAbsolutePath(), getTargetAlgorithmExecutionCommandAsString(execConfig, runConfig));
+							
+								if(outputQueue.size() > 0)
+								{
+									log.error("The last {} lines of output we saw were:", outputQueue.size());
+									
+									for(String s : outputQueue)
+									{
+										log.error("> "+s);
+									}
+								} else
+								{
+									log.debug("No output on standard out detected");
+								}
+								
+								
+							
+						default:
+						
+					}
+					
+					outputQueue.clear();
+
+					read.close();
+
+					stdErrorDone.acquireUninterruptibly();
+				} finally
+				{
+					//Close the listening socket
+					if(serverSocket != null)
+					{
+						serverSocket.close();
+					}
+					threadPoolExecutor.shutdownNow();
+					try {
+						threadPoolExecutor.awaitTermination(24, TimeUnit.HOURS);
+					} catch (InterruptedException e) {
+						Thread.currentThread().interrupt();
 					}
 				}
-				
-				
-				switch(this.getRunResult())
-				{
-				
-				
-				case ABORT:
-				case CRASHED:
-					
-				
-						log.error( "Failed Run Detected Call: cd \"{}\" " + commandSeparator + "  {} ",new File(execConfig.getAlgorithmExecutionDirectory()).getAbsolutePath(), getTargetAlgorithmExecutionCommandAsString(execConfig, runConfig));
-					
-						log.error("Failed Run Detected output last {} lines", outputQueue.size());
-						
-						for(String s : outputQueue)
-						{
-							log.error("> "+s);
-						}
-						log.error("Output complete");
-						
-					
-				default:
-					
-				}
-				
-				outputQueue.clear();
-				
-				
-				read.close();
 			
 				
 				
 				
-				stdErrorDone.acquireUninterruptibly();
-				
-				threadPoolExecutor.shutdownNow();
-				//Close the listening socket
-				serverSocket.close();
-				try {
-					threadPoolExecutor.awaitTermination(24, TimeUnit.HOURS);
-				} catch (InterruptedException e) {
-					Thread.currentThread().interrupt();
-				}
 	
 				runObserver.currentStatus(Collections.singletonList(new KillableWrappedAlgorithmRun(this)));
 				log.debug("Run {} is completed", this);
@@ -496,7 +615,7 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 			}
 		} catch (IOException e1) {
 			//String execCmd = getTargetAlgorithmExecutionCommandAsString(execConfig,runConfig);
-			log.error( "Failed Run Detected (IOException) Call: cd \"{}\" " + commandSeparator + "  {} ",new File(execConfig.getAlgorithmExecutionDirectory()).getAbsolutePath(), getTargetAlgorithmExecutionCommandAsString(execConfig, runConfig));
+			log.error( "The following algorithm call failed: cd \"{}\" " + COMMAND_SEPERATOR + "  {} ",new File(execConfig.getAlgorithmExecutionDirectory()).getAbsolutePath(), getTargetAlgorithmExecutionCommandAsString(execConfig, runConfig));
 			throw new TargetAlgorithmAbortException(e1);
 			//throw new IllegalStateException(e1);
 		}
@@ -506,6 +625,7 @@ public class CommandLineAlgorithmRun extends AbstractAlgorithmRun {
 		
 	}
 	
+		
 	/**
 	 * Processes all the output of the target algorithm
 	 * 
@@ -533,7 +653,7 @@ outerloop:
 						
 						if(line == null)
 						{
-							log.debug("Process has ended");
+							log.trace("Process has ended");
 							processEnded.set(true);
 							break outerloop;
 						}
@@ -583,9 +703,9 @@ outerloop:
 					
 					if(!read)
 					{
-						if(++i % 200 == 0)
+						if(++i % 12000 == 0)
 						{
-							log.debug("Slept for 5 second waiting for pid {}  && {} " ,getPID(p), matchFound);
+							log.trace("Slept for 5 minutes waiting for pid {}  &&  (matching line found?: {} ) " ,getPID(p), matchFound);
 						}
 						Thread.sleep(25);
 					}
@@ -597,7 +717,7 @@ outerloop:
 				
 				if(!processEnded.get())
 				{
-					log.debug("IO Exception occurred while processing runs");
+					log.trace("IO Exception occurred while processing runs");
 				}
 				
 			} catch (InterruptedException e) {
@@ -624,7 +744,7 @@ outerloop:
 		
 		if(options.logAllCallStrings)
 		{
-			log.info( "Call (with token {}) : cd \"{}\" " + commandSeparator + "  {} ", token, new File(execConfig.getAlgorithmExecutionDirectory()).getAbsolutePath(), getTargetAlgorithmExecutionCommandAsString(execConfig, runConfig));
+			log.info( "Call (with token {}) : cd \"{}\" " + COMMAND_SEPERATOR + "  {} ", token, new File(execConfig.getAlgorithmExecutionDirectory()).getAbsolutePath(), getTargetAlgorithmExecutionCommandAsString(execConfig, runConfig));
 		}
 		
 		
@@ -642,9 +762,11 @@ outerloop:
 		}
 		
 		envpList.add(CONCURRENT_TASK_ID + "=" + token);
-		
+		envpList.add(envVariableForChildren + "=" + uuid.toString());  
 		String[] envp = envpList.toArray(new String[0]);
 		Process proc = Runtime.getRuntime().exec(execCmdArray,envp, new File(execConfig.getAlgorithmExecutionDirectory()));
+		
+		log.debug("Process for {} started with pid: {} (Environment Variable: {})", this.runConfig, getPID(proc), uuid);
 		return proc;
 	}
 	
@@ -653,7 +775,7 @@ outerloop:
 	 * Gets the execution command string
 	 * @return string containing command
 	 */
-	private static String[] getTargetAlgorithmExecutionCommand(AlgorithmExecutionConfig execConfig, RunConfig runConfig)
+	private String[] getTargetAlgorithmExecutionCommand(AlgorithmExecutionConfig execConfig, RunConfig runConfig)
 	{
 
 				
@@ -672,16 +794,18 @@ outerloop:
 		
 		StringFormat f = StringFormat.NODB_SYNTAX;
 		
+		final String valueDelimiter = (options.paramArgumentsContainQuotes) ?  f.getValueDelimeter() : "";
+		
 		for(String key : runConfig.getParamConfiguration().getActiveParameters() )
 		{
-			
-			
 			if(!f.getKeyValueSeperator().equals(" ") || !f.getGlue().equals(" "))
 			{
 				throw new IllegalStateException("Key Value seperator or glue is not a space, and this means the way we handle this logic won't work currently");
 			}
 			list.add(f.getPreKey() + key);
-			list.add(f.getValueDelimeter() + runConfig.getParamConfiguration().get(key)  + f.getValueDelimeter());	
+			
+			
+			list.add(valueDelimiter + runConfig.getParamConfiguration().get(key)  + valueDelimiter);	
 			
 		}
 		
@@ -750,12 +874,14 @@ outerloop:
 	
 	
 	/**
-	 *	Process a single line of the output looking for a matching line (e.g. Result for ParamILS: ...)
+	 *	Process a single line of the output looking for a matching line (e.g. Result of algorithm run: ...)
 	 *	@param line of program output
 	 */
 	public boolean processLine(String line)
 	{
 		Matcher matcher = pattern.matcher(line);
+		
+		Matcher matcher2 = oldPattern.matcher(line);
 		String rawResultLine = "[No Matching Output Found]";
 		
 		if(options.logAllProcessOutput)
@@ -764,7 +890,7 @@ outerloop:
 		}
 		
 
-		if (matcher.find())
+		if (matcher.find() || matcher2.find())
 		{
 		
 			String fullLine = line.trim();
@@ -772,7 +898,7 @@ outerloop:
 			try
 			{
 			
-				String acExecResultString = line.substring(matcher.end()).trim();
+				String acExecResultString = line.substring(line.indexOf(":")+1).trim();
 				
 				String[] results = acExecResultString.split(",");
 				for(int i=0; i < results.length; i++)
@@ -915,11 +1041,60 @@ outerloop:
 	{
 		return input.replaceAll("%pid", String.valueOf(pid));
 	}
+	
+	
+	AtomicBoolean killPreviouslyCalled = new AtomicBoolean(false);
 	private void killProcess(Process p)
 	{
+	
+		if(killPreviouslyCalled.getAndSet(true))
+		{
+			return;
+		} else
+		{
+			outstandingRuns.remove(new Pair<CommandLineAlgorithmRun, Process>(this,p));
+		}
+		
 		
 		try 
 		{
+			
+			if(options.pgEnvKillCommand != null)
+			{
+				try {
+					
+					String killEnvCmd = options.pgEnvKillCommand + " " + envVariableForChildren + " " + uuid.toString();
+					Process p2 = Runtime.getRuntime().exec(SplitQuotedString.splitQuotedString(killEnvCmd));
+					
+					BufferedReader read = new BufferedReader(new InputStreamReader(p2.getErrorStream()));
+					String line = null;
+					
+					while((line = read.readLine()) != null)
+					{
+						log.trace("Kill environment {} error> {}", uuid.toString(), line);
+					}
+					read.close();
+					read = new BufferedReader(new InputStreamReader(p2.getInputStream()));
+					line = null;
+					
+					while((line = read.readLine()) != null)
+					{
+						log.trace("Kill environment {}  output> {}", uuid.toString(), line);
+					}
+					
+					read.close();
+					
+					
+					p2.destroy();
+				} catch(IOException e)
+				{
+					
+					log.error("Error while executing {} execute Kill Environment Command",e);
+					
+				}
+			}
+			
+			
 			if(exited(p))
 			{
 				return;
@@ -933,54 +1108,56 @@ outerloop:
 					
 					if(pid > 0)
 					{
-						
-						
-						
 						String command = replacePid(options.pgNiceKillCommand,pid);
-						log.debug("Trying to send SIGTERM to process group id: {} with command \"{}\"", pid,command);
+						log.trace("Trying to send SIGTERM to process group id: {} with command \"{}\"", pid,command);
 						try {
 							
 							Process p2 = Runtime.getRuntime().exec(SplitQuotedString.splitQuotedString(command));
 							
+							
 							BufferedReader read = new BufferedReader(new InputStreamReader(p2.getErrorStream()));
 							String line = null;
 							
 							while((line = read.readLine()) != null)
 							{
-								log.debug("Kill error output> {}", line);
+								log.trace("Kill error output> {}", line);
 							}
+							read.close();
 							read = new BufferedReader(new InputStreamReader(p2.getInputStream()));
 							line = null;
 							
 							
 							while((line = read.readLine()) != null)
 							{
-								log.debug("Kill output Input> {}", line);
+								log.trace("Kill output Input> {}", line);
 							}
 							
-							
+							read.close();
 							int retValPGroup = p2.waitFor();
 							
 							if(retValPGroup > 0)
 							{
-								log.debug("SIGTERM to process group failed with error code {}", retValPGroup);
+								log.trace("SIGTERM to process group failed with error code {}", retValPGroup);
 								Process p3 = Runtime.getRuntime().exec(SplitQuotedString.splitQuotedString(replacePid(options.procNiceKillCommand,pid)));
 								int retVal = p3.waitFor();
 								
+								p3.destroy();
 								if(retVal > 0)
 								{
 									Object[] args = {  pid,retVal};
-									log.debug("SIGTERM to process id: {} attempted failed with return code {}",args);
+									log.trace("SIGTERM to process id: {} attempted failed with return code {}",args);
 								} else
 								{
-									log.debug("SIGTERM delivered successfully to process id: {}", pid, pid);
+									log.trace("SIGTERM delivered successfully to process id: {}", pid, pid);
 								}
 									
 								
+								
 							} else
 							{
-								log.debug("SIGTERM delivered successfully to process group id: {} ", pid);
+								log.trace("SIGTERM delivered successfully to process group id: {} ", pid);
 							}
+							p2.destroy();
 						} catch (IOException e) {
 							log.error("Couldn't SIGTERM process or process group ", e);
 						}
@@ -1008,14 +1185,14 @@ outerloop:
 							
 						}
 												
-						log.debug("Trying to send SIGKILL to process group id: {}", pid);
+						log.trace("Trying to send SIGKILL to process group id: {}", pid);
 						try {
 							Process p2 = Runtime.getRuntime().exec(SplitQuotedString.splitQuotedString(replacePid(options.pgForceKillCommand,pid)));
 							int retVal = p2.waitFor();
 							
 							if(retVal > 0)
 							{
-								log.debug("SIGKILL to pid: {} attempted failed with return code {}",pid, retVal);
+								log.trace("SIGKILL to pid: {} attempted failed with return code {}",pid, retVal);
 								
 								Process p3 = Runtime.getRuntime().exec(SplitQuotedString.splitQuotedString(replacePid(options.procForceKillCommand,pid)));
 								int retVal3 = p3.waitFor();
@@ -1023,17 +1200,18 @@ outerloop:
 								if(retVal3 > 0)
 								{
 									Object[] args = {  pid,retVal};
-									log.debug("SIGKILL to process id: {} attempted failed with return code {}",args);
+									log.trace("SIGKILL to process id: {} attempted failed with return code {}",args);
 								} else
 								{
-									log.debug("SIGKILL delivered successfully to process id: {}", pid, pid);
+									log.trace("SIGKILL delivered successfully to process id: {}", pid, pid);
 								}
 								
-								
+								p3.destroy();
 							} else
 							{
-								log.debug("SIGKILL delivered successfully to pid: {} ", pid);
+								log.trace("SIGKILL delivered successfully to pid: {} ", pid);
 							}
+							p2.destroy();
 						} catch (IOException e) {
 							log.error("Couldn't SIGKILL process or process group ", e);
 							
