@@ -71,7 +71,7 @@ public class NoRunsForManyChallengesEvent extends AbstractTerminationCondition i
 	public String getTerminationReason() {
 		if(haveToStop())
 		{
-			return "Too many challenges have been attempted without a successful run (" +  attemptedChallengesSinceLastRun +  ") has been reached";
+			return "too many challenges have been attempted without a successful run (" +  attemptedChallengesSinceLastRun +  ") has been reached.";
 		} else
 		{
 			return "";
