@@ -14,12 +14,14 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import ca.ubc.cs.beta.aclib.acquisitionfunctions.AcquisitionFunction;
+import ca.ubc.cs.beta.aclib.algorithmrun.AlgorithmRun;
 import ca.ubc.cs.beta.aclib.configspace.ParamConfiguration;
 import ca.ubc.cs.beta.aclib.configspace.ParamConfigurationSpace;
 import ca.ubc.cs.beta.aclib.configspace.tracking.ParamConfigurationOriginTracker;
 import ca.ubc.cs.beta.aclib.eventsystem.EventManager;
 import ca.ubc.cs.beta.aclib.initialization.InitializationProcedure;
 import ca.ubc.cs.beta.aclib.misc.associatedvalue.ParamWithEI;
+import ca.ubc.cs.beta.aclib.misc.cputime.CPUTime;
 import ca.ubc.cs.beta.aclib.misc.watch.AutoStartStopWatch;
 import ca.ubc.cs.beta.aclib.misc.watch.StopWatch;
 import ca.ubc.cs.beta.aclib.model.builder.AdaptiveCappingModelBuilder;
@@ -32,6 +34,7 @@ import ca.ubc.cs.beta.aclib.model.data.SanitizedModelData;
 import ca.ubc.cs.beta.aclib.probleminstance.ProblemInstance;
 import ca.ubc.cs.beta.aclib.random.SeedableRandomPool;
 import ca.ubc.cs.beta.aclib.runhistory.RunHistory;
+import ca.ubc.cs.beta.aclib.runhistory.RunHistoryHelper;
 import ca.ubc.cs.beta.aclib.runhistory.TeeRunHistory;
 import ca.ubc.cs.beta.aclib.runhistory.ThreadSafeRunHistory;
 import ca.ubc.cs.beta.aclib.runhistory.ThreadSafeRunHistoryWrapper;
@@ -73,8 +76,8 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 	
 	private final RunHistory modelRunHistory;
 
-	public SequentialModelBasedAlgorithmConfiguration(SMACOptions smacConfig, List<ProblemInstance> instances, TargetAlgorithmEvaluator algoEval, AcquisitionFunction ei, StateFactory sf, ParamConfigurationSpace configSpace, InstanceSeedGenerator instanceSeedGen, ParamConfiguration initialConfiguration, EventManager eventManager, ThreadSafeRunHistory rh, SeedableRandomPool pool, CompositeTerminationCondition termCond, ParamConfigurationOriginTracker configTracker, InitializationProcedure initProc, RunHistory modelRH) {
-		super(smacConfig, instances, algoEval,sf, configSpace, instanceSeedGen, initialConfiguration, eventManager, rh, pool, termCond, configTracker,initProc);
+	public SequentialModelBasedAlgorithmConfiguration(SMACOptions smacConfig, List<ProblemInstance> instances, TargetAlgorithmEvaluator algoEval, AcquisitionFunction ei, StateFactory sf, ParamConfigurationSpace configSpace, InstanceSeedGenerator instanceSeedGen, ParamConfiguration initialConfiguration, EventManager eventManager, ThreadSafeRunHistory rh, SeedableRandomPool pool, CompositeTerminationCondition termCond, ParamConfigurationOriginTracker configTracker, InitializationProcedure initProc, RunHistory modelRH, CPUTime cpuTime) {
+		super(smacConfig, instances, algoEval,sf, configSpace, instanceSeedGen, initialConfiguration, eventManager, rh, pool, termCond, configTracker,initProc, cpuTime);
 		numPCA = smacConfig.numPCA;
 		logModel = smacConfig.randomForestOptions.logModel;
 		this.smacConfig = smacConfig;
@@ -82,7 +85,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		
 		if(modelRH.getAlgorithmRunData().size() > 0)
 		{
-			log.info("Model warmstart payload detected with {} runs ", modelRH.getAlgorithmRunData().size());
+			log.debug("Model warmstart payload detected with {} runs ", modelRH.getAlgorithmRunData().size());
 		}
 		this.modelRunHistory = modelRH;
 	 
@@ -91,11 +94,11 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 	
 	protected double freeMemoryAfterGC()
 	{
-		log.debug("Running System Garbage Collector");
+		log.trace("Running System Garbage Collector");
 		System.gc();
 		
 		double freeMemory = ((double) Runtime.getRuntime().freeMemory() / (double) Runtime.getRuntime().totalMemory());
-		log.debug("Free memory {} %", freeMemory);
+		log.trace("Free memory {} %", freeMemory);
 		return freeMemory;
 	}
 	
@@ -112,7 +115,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		runHistory = modelRunHistory;
 		
 		
-		if(runHistory.getAlgorithmRuns().size() == 0)
+		if(runHistory.getAlgorithmRunsExcludingRedundant().size() == 0)
 		{
 			throw new IllegalStateException("Expected one run to be done before building model");
 		}
@@ -125,7 +128,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 			{
 				subsamplePercentage *= options.randomForestOptions.subsamplePercentage;
 				Object[] args = { getIteration(), freeMemory, subsamplePercentage};
-				log.info("Iteration {} : Free memory too low ({}) subsample percentage now {} ", args);
+				log.debug("Iteration {} : Free memory too low ({}) subsample percentage now {} ", args);
 			}
 
 		} else
@@ -183,8 +186,11 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 			usedInstanceIdxs[j] = runInstancesIdx.get(j);
 		}
 		
-		double[] runResponseValues = runHistory.getRunResponseValues();
-		boolean[] censored = runHistory.getCensoredFlagForRuns();
+		
+		List<AlgorithmRun> runs = runHistory.getAlgorithmRunsExcludingRedundant();
+		double[] runResponseValues = RunHistoryHelper.getRunResponseValues(runs, runHistory.getRunObjective());
+		boolean[] censored = RunHistoryHelper.getCensoredEarlyFlagForRuns(runs);
+		
 		if(smacConfig.mbOptions.maskCensoredDataAsKappaMax)
 		{
 			for(int j=0; j < runResponseValues.length; j++)
@@ -202,12 +208,12 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 			// but matlab sure does
 			if(runResponseValues[j] >= options.scenarioConfig.algoExecOptions.cutoffTime)
 			{	
-				runResponseValues[j] = options.scenarioConfig.algoExecOptions.cutoffTime * options.scenarioConfig.intraInstanceObj.getPenaltyFactor();
+				runResponseValues[j] = options.scenarioConfig.algoExecOptions.cutoffTime * options.scenarioConfig.getIntraInstanceObjective().getPenaltyFactor();
 			}
 		}
 	
 		//=== Sanitize the data.
-		sanitizedData = new PCAModelDataSanitizer(instanceFeatureMatrix, thetaMatrix, numPCA, runResponseValues, usedInstanceIdxs, logModel, runHistory.getParameterConfigurationInstancesRanByIndex(), runHistory.getCensoredFlagForRuns(), configSpace);
+		sanitizedData = new PCAModelDataSanitizer(instanceFeatureMatrix, thetaMatrix, numPCA, runResponseValues, usedInstanceIdxs, logModel, runHistory.getParameterConfigurationInstancesRanByIndex(), censored, configSpace);
 		
 		
 		if(smacConfig.mbOptions.maskCensoredDataAsUncensored)
@@ -229,7 +235,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		preparedForest = null;
 		if(options.adaptiveCapping)
 		{
-			mb = new AdaptiveCappingModelBuilder(sanitizedData, smacConfig.randomForestOptions, pool.getRandom("RANDOM_FOREST_BUILDING_PRNG"), smacConfig.mbOptions.imputationIterations, smacConfig.scenarioConfig.algoExecOptions.cutoffTime, smacConfig.scenarioConfig.intraInstanceObj.getPenaltyFactor(), subsamplePercentage);
+			mb = new AdaptiveCappingModelBuilder(sanitizedData, smacConfig.randomForestOptions, pool.getRandom("RANDOM_FOREST_BUILDING_PRNG"), smacConfig.mbOptions.imputationIterations, smacConfig.scenarioConfig.algoExecOptions.cutoffTime, smacConfig.scenarioConfig.getIntraInstanceObjective().getPenaltyFactor(), subsamplePercentage);
 		} else
 		{
 			//mb = new HashCodeVerifyingModelBuilder(sanitizedData,smacConfig.randomForestOptions, runHistory);
@@ -240,7 +246,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		forest = mb.getRandomForest();
 		preparedForest = mb.getPreparedRandomForest();
 	
-		log.info("Random Forest Built");
+		log.debug("Random Forest Built");
 	}
 	
 	//private int selectionCount = 0;
@@ -250,7 +256,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		AutoStartStopWatch t = new AutoStartStopWatch();
 		List<ParamConfiguration> eichallengers = selectChallengersWithEI(smacConfig.numberOfChallengers);
 		
-		log.info("Selecting {} challengers based on EI took {} seconds", eichallengers.size(), t.stop() / 1000.0);
+		log.debug("Selecting {} challengers based on EI took {} seconds", eichallengers.size(), t.stop() / 1000.0);
 		
 		List<ParamConfiguration> randomChallengers = new ArrayList<ParamConfiguration>(eichallengers.size());
 		
@@ -260,7 +266,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		{
 			randomChallengers.add(configSpace.getRandomConfiguration(configSpaceRandomInterleave));
 		}
-		log.debug("Generating {} Random Configurations took {} seconds", eichallengers.size(),  t.stop()/1000.0 );
+		log.trace("Generating {} Random Configurations took {} seconds", eichallengers.size(),  t.stop()/1000.0 );
 		
 		//=== Interleave the EI and random challengers.
 		List<ParamConfiguration> challengers = new ArrayList<ParamConfiguration>(eichallengers.size()*2);
@@ -280,7 +286,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		
 		if(SELECT_CONFIGURATION_SYNC_DEBUGGING)
 		{
-			log.debug("Final Selected Challengers Configurations Hash Code {}", matlabHashCode(configArrayToDebug));
+			log.trace("Final Selected Challengers Configurations Hash Code {}", matlabHashCode(configArrayToDebug));
 		}
 		
 		return challengers;
@@ -290,7 +296,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 	public List<ParamConfiguration> selectChallengersWithEI(int numChallengers)
 	{
 		Set<ProblemInstance> instanceSet = new HashSet<ProblemInstance>();
-		instanceSet.addAll(runHistory.getInstancesRan(incumbent));
+		instanceSet.addAll(runHistory.getProblemInstancesRan(incumbent));
 		
 		//=== Get predictions for all configurations we have run thus far.
 		List<ParamConfiguration> paramConfigs = runHistory.getAllParameterConfigurationsRan();
@@ -299,7 +305,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		double[] predvar = predictions[1];
 
 		double[][] tmp_predictions = transpose(applyMarginalModel(Collections.singletonList(incumbent)));
-		log.info("Prediction for incumbent: {} +/- {} (in log space if logModel=true)", tmp_predictions[0][0], tmp_predictions[1][0]);
+		log.trace("Prediction for incumbent: {} +/- {} (in log space if logModel=true)", tmp_predictions[0][0], tmp_predictions[1][0]);
 		
 		double fmin = runHistory.getEmpiricalCost(incumbent, instanceSet, smacConfig.scenarioConfig.algoExecOptions.cutoffTime);
 		//=== Get the empirical cost into log space if the model gives log predictions. 
@@ -314,7 +320,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 			adjusted_fmin = Math.log10(adjusted_fmin);
 			Object[] args = { getIteration(), fmin, adjusted_fmin};
 			
-			log.info("Optimizing EI at valdata.iteration {}. fmin: {}, fmin (accounting for minimum response value): {}", args);
+			log.trace("Optimizing EI at valdata.iteration {}. fmin: {}, fmin (accounting for minimum response value): {}", args);
 			/*
 			if(tmp_predictions[1][0] > Math.pow(10, -13))
 			{
@@ -324,7 +330,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 				{
 					if(run.getRunConfig().getParamConfiguration().equals(incumbent))
 					{
-						log.info("Instance {} Runtime {}:", run.getRunConfig().getProblemInstanceSeedPair().getInstance().getInstanceID(), run.getRuntime());
+						log.debug("Instance {} Runtime {}:", run.getRunConfig().getProblemInstanceSeedPair().getInstance().getInstanceID(), run.getRuntime());
 					}
 				}
 				
@@ -335,7 +341,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 			*/
 		} else
 		{
-			log.info("Optimizing EI at valdata.iteration {}. fmin: {}", getIteration(), fmin);
+			log.debug("Optimizing EI at valdata.iteration {}. fmin: {}", getIteration(), fmin);
 		}
 		
 		//=== Compute EI of these configurations (as given by predmean,predvar)
@@ -345,7 +351,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		
 
 		watch.stop();
-		log.info("Compute negEI for all conf. seen at valdata.iteration {}: took {} s",getIteration(), ((double) watch.time()) / 1000.0 );
+		log.debug("Compute negEI for all conf. seen at valdata.iteration {}: took {} s",getIteration(), ((double) watch.time()) / 1000.0 );
 
 		//=== Put these EIs into a map for each configuration.
 		Map<ParamConfiguration, double[]> configPredMeanVarEIMap = new LinkedHashMap<ParamConfiguration, double[]>();
@@ -392,10 +398,10 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 			configPredMeanVarEIMap.put(lsResult.getValue(), val);
 			watch.stop();
 			Object[] args = {i+1,((double) watch.time()/1000.0),min_neg};
-			log.debug("LS {} took {} seconds and yielded neg log EI {}",args);
+			log.trace("LS {} took {} seconds and yielded neg log EI {}",args);
 		}
 		
-		log.info("{} Local Searches took {} seconds in total ", numberOfSearches, stpWatch.stop()  / 1000.0 );
+		log.trace("{} Local Searches took {} seconds in total ", numberOfSearches, stpWatch.stop()  / 1000.0 );
 		//=== Get into array format for debugging.
 		double[][] configArrayToDebug = new double[bestResults.size()][];
 		int j=0; 
@@ -421,7 +427,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 			randomConfigs.add(configSpace.getRandomConfiguration(configSpaceEIRandom));
 		} 
 		
-		log.debug("Generating {} Random Configurations for EI took {} (s)", numberOfRandomConfigsInEI, t.stop() / 1000.0);
+		log.trace("Generating {} Random Configurations for EI took {} (s)", numberOfRandomConfigsInEI, t.stop() / 1000.0);
 		
 		t = new AutoStartStopWatch();
 		double[][] randomConfigToDebug = new double[randomConfigs.size()][];
@@ -431,17 +437,17 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		}
 		if(SELECT_CONFIGURATION_SYNC_DEBUGGING &&  log.isDebugEnabled())
 		{
-			log.debug("Local Search Selected Random Configs Hash Code: {}", matlabHashCode(randomConfigToDebug));
+			log.trace("Local Search Selected Random Configs Hash Code: {}", matlabHashCode(randomConfigToDebug));
 		}
 		//=== Compute EI for the random configs.		
 		predictions = transpose(applyMarginalModel(randomConfigs));
 		predmean = predictions[0];
 		predvar = predictions[1];
 		
-		log.debug("Prediction for Random Configurations took {} (s)", t.stop() / 1000.0);
+		log.trace("Prediction for Random Configurations took {} (s)", t.stop() / 1000.0);
 		t = new AutoStartStopWatch();
 		double[] expectedImprovementOfRandoms = ei.computeAcquisitionFunctionValue(fmin, predmean, predvar);
-		log.debug("EI Calculation for Random Configurations took {} (s)", t.stop() / 1000.0);
+		log.trace("EI Calculation for Random Configurations took {} (s)", t.stop() / 1000.0);
 		t = new AutoStartStopWatch();
 		for(int i=0; i <  randomConfigs.size(); i++)
 		{
@@ -449,7 +455,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 			configPredMeanVarEIMap.put(randomConfigs.get(i), val);
 		}
 		
-		log.debug("Map Insertion for Random Configurations took {} (s)", t.stop() / 1000.0);
+		log.trace("Map Insertion for Random Configurations took {} (s)", t.stop() / 1000.0);
 		
 		
 		//=== Add random configs to LS configs.
@@ -461,9 +467,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		for(ParamWithEI eic : bestResults)
 		{
 			configArrayToDebug[j++] = eic.getValue().toValueArray();
-		}
-		//log.debug("Local Search Selected Configurations & Random Configs Hash Code: {}", matlabHashCode(configArrayToDebug));
-		
+		}		
 		
 		/*
 		if(RoundingMode.ROUND_NUMBERS_FOR_MATLAB_SYNC)
@@ -492,7 +496,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		{
 			double[] meanvar = configPredMeanVarEIMap.get(bestResults.get(i).getValue());
 			Object[] args = {i+1, meanvar[0], Math.sqrt(meanvar[1]), -meanvar[2]}; 
-			log.info("Challenger {} predicted {} +/- {}, expected improvement {}",args);
+			log.trace("Challenger {} predicted {} +/- {}, expected improvement {}",args);
 		}
 		
 		//=== Make result list of configurations. 
@@ -506,7 +510,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 		for(int i = 0; i < Math.min(25, bestResults.size()); i++)
 		{
 			ParamWithEI eic = bestResults.get(i);
-			configTracker.addConfiguration(eic.getValue(), "Model-Builder-" +this.getIteration(),"EIMethod="+options.expFunc, "EI=" + eic.getAssociatedValue() , "firstArg(k)=" + fmin, "ModelVersion=" + this.getIteration() , "ModelPoints=" + this.runHistory.getAlgorithmRuns().size());
+			configTracker.addConfiguration(eic.getValue(), "Model-Builder-" +this.getIteration(),"EIMethod="+options.expFunc, "EI=" + eic.getAssociatedValue() , "firstArg(k)=" + fmin, "ModelVersion=" + this.getIteration() , "ModelPoints=" + this.runHistory.getAlgorithmRunsExcludingRedundant().size());
 		}
 		
 		
@@ -561,7 +565,7 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 			double[][] cArray = {c.toValueArray()};
 			int LSHashCode = matlabHashCode(cArray);
 			
-			if(SELECT_CONFIGURATION_SYNC_DEBUGGING) log.debug("Local Search HashCode: {}", LSHashCode);
+			if(SELECT_CONFIGURATION_SYNC_DEBUGGING) log.trace("Local Search HashCode: {}", LSHashCode);
 			
 			//=== Get neighbourhood of current options and compute EI for all of it.
 			List<ParamConfiguration> neighbourhood = c.getNeighbourhood(configRandLS, options.scenarioConfig.algoExecOptions.paramFileDelegate.continuousNeighbours);
@@ -611,15 +615,10 @@ public class SequentialModelBasedAlgorithmConfiguration extends
 				double[] var = predictions[1];
 				
 				eiVal = ei.computeAcquisitionFunctionValue(fmin_sample, mean, var);
-				Object[] args = {eiVal[0], mean[0], var[0]};
-				log.trace("Expected improvement for next step is {} mean={}, var={}",args);
-				
-				
-				
 			}
 		}
 		
-		log.debug("Local Search took {} steps", localSearchSteps);
+		log.trace("Local Search took {} steps", localSearchSteps);
 		
 		return incumbentEIC;
 	}
